# -*- coding: utf-8 -*-

from django.test import tag
from django.core import mail
from django.conf import settings
from django.urls import reverse
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.core.files.uploadedfile import UploadedFile

from guichet.tests import CustomTestCase
from accounts.models import User, Group, Token, AssociateToken
from guichet.models import (
    Event,
    Manager,
    GroupManager,
    Ticket,
    Registration,
    Option,
    Info_Sup,
    Info_Sup_Int,
    Info_Sup_File,
)
from paypal.standard.ipn.models import PayPalIPN

from .utils import createLdapUser, test_user, createExteUser, loginUser

from datetime import timedelta

import os


class associationTest(CustomTestCase):
    def setUp(self):
        createExteUser("exte@test.inpt.fr", "passwd", "test", "exte")
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))

    @tag("exte", "fast")
    def test_nullassociationForm(self):
        form = {
            "login_inp_net": "",
            "associate": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 200)
        self.assertFormError(response, "associateform", "login_inp_net", "Ce champ est obligatoire.")

    @tag("exte", "fast")
    def test_removeAssociationWithoutDelay(self):
        form = {
            "login_inp_net": "testwo",
            "associate": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertRedirects(response, reverse("profile"))
        response_2 = self.client.post(reverse("associate_del"))
        self.assertRedirects(response_2, reverse("profile"))
        self.assertTrue(AssociateToken.objects.filter(user=User.objects.get(email="exte@test.inpt.fr")).exists())

    @tag("exte", "fast", "local")
    def test_removeAssociationWithDelay(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        token.created_at = timezone.now() - timedelta(hours=50)
        token.save()
        response = self.client.post(reverse("associate_del"))
        self.assertRedirects(response, reverse("profile"))
        self.assertFalse(AssociateToken.objects.filter(user=User.objects.get(email="exte@test.inpt.fr")).exists())

    @tag("exte", "fast", "local")
    def test_invalidate(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(reverse("associate_account"), {"invalidate": "True"})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).invalide)

    @tag("exte", "fast", "local")
    def test_invalidateUserLink(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(
            reverse("associate_account"), {"invalidate": "True", "part": "user", "token": token.token_user}
        )
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).invalide)

    @tag("exte", "fast", "local")
    def test_invalidateLdapLink(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(
            reverse("associate_account"), {"invalidate": "True", "part": "ldap", "token": token.token_user_ldap}
        )
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).invalide)

    @tag("exte", "fast", "local")
    def test_invalidateUserLinkwithValidate(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
            invalide=True,
        )
        token.save()
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).invalide)
        response = self.client.get(reverse("associate_account"), {"part": "user", "token": token.token_user})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertFalse(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user)

    @tag("exte", "fast", "local")
    def test_invalidateLdapLinkwithValidate(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
            invalide=True,
        )
        token.save()
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).invalide)
        response = self.client.get(reverse("associate_account"), {"part": "ldap", "token": token.token_user_ldap})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertFalse(
            AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user_ldap
        )

    @tag("exte", "fast", "local")
    def test_validateLdapThenUser(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(reverse("associate_account"), {"part": "ldap", "token": token.token_user_ldap})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(
            AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user_ldap
        )
        response2 = self.client.get(reverse("associate_account"), {"part": "user", "token": token.token_user})
        self.assertRedirects(response2, reverse("home"), fetch_redirect_response=False)
        self.assertFalse(User.objects.filter(email="exte@test.inpt.fr").exists())
        self.assertEqual(len(User.objects.filter(login_inp_net="testwo")), 1)
        self.assertFalse(User.objects.get(login_inp_net="testwo").has_usable_password())
        self.assertFalse(AssociateToken.objects.all().exists())

    @tag("exte", "fast", "local")
    def test_validateUserThenLdap(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(reverse("associate_account"), {"part": "user", "token": token.token_user})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user)
        response2 = self.client.get(reverse("associate_account"), {"part": "ldap", "token": token.token_user_ldap})
        self.assertRedirects(response2, reverse("home"), fetch_redirect_response=False)

        self.assertFalse(User.objects.filter(email="exte@test.inpt.fr").exists())
        self.assertEqual(len(User.objects.filter(login_inp_net="testwo")), 1)
        self.assertFalse(User.objects.get(login_inp_net="testwo").has_usable_password())
        self.assertFalse(AssociateToken.objects.all().exists())

    @tag("exte", "fast", "local")
    def test_validateUserThenLdapConnection(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(reverse("associate_account"), {"part": "user", "token": token.token_user})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user)
        createLdapUser("testwo")
        response2 = self.client.get(reverse("associate_account"), {"part": "ldap", "token": token.token_user_ldap})
        self.assertRedirects(response2, reverse("home"), fetch_redirect_response=False)

        self.assertTrue(User.objects.filter(email="exte@test.inpt.fr").exists())
        self.assertTrue(User.objects.get(email="exte@test.inpt.fr").has_usable_password())
        self.assertEqual(len(User.objects.filter(login_inp_net="testwo")), 1)

    @tag("exte", "fast", "local")
    def test_validateLdapThenLdapConnection(self):
        token = AssociateToken(
            user=User.objects.get(email="exte@test.inpt.fr"),
            user_ldap="testwo",
            token_user=get_random_string(length=32),
            token_user_ldap=get_random_string(length=32),
        )
        token.save()
        response = self.client.get(reverse("associate_account"), {"part": "ldap", "token": token.token_user_ldap})
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)
        self.assertTrue(
            AssociateToken.objects.get(user=User.objects.get(email="exte@test.inpt.fr")).validation_user_ldap
        )
        createLdapUser("testwo")
        response2 = self.client.get(reverse("associate_account"), {"part": "user", "token": token.token_user})
        self.assertRedirects(response2, reverse("home"), fetch_redirect_response=False)

        self.assertTrue(User.objects.filter(email="exte@test.inpt.fr").exists())
        self.assertTrue(User.objects.get(email="exte@test.inpt.fr").has_usable_password())
        self.assertEqual(len(User.objects.filter(login_inp_net="testwo")), 1)


class profileTest(CustomTestCase):
    def setUp(self):
        createLdapUser("testone")
        createExteUser("exte@test.inpt.fr", "passwd", "test", "exte")

    @tag("nologin", "fast", "local")
    def test_noUser(self):
        response = self.client.get(reverse("profile"))
        self.assertRedirects(response, reverse("login") + "?next=" + reverse("profile"))

    @tag("ldap", "local")
    def test_LdapUserGet(self):
        for login in test_user:
            createLdapUser(login)
            self.assertTrue(loginUser(self.client, login))
            response = self.client.get(reverse("profile"))
            self.assertTemplateUsed(response, "accounts/profile.html")
            if test_user[login]["AE"]:
                self.assertContains(response, "Accès aux places réservées aux cotisants de l’AE", status_code=200)
            else:
                self.assertNotContains(response, "Accès aux places réservées aux cotisants de l’AE", status_code=200)
            for ecoleid in ["n7", "a7", "ensat"]:
                if test_user[login]["ecole"] == ecoleid:
                    self.assertContains(
                        response, "Accès aux places réservées aux étudiants de l’" + ecoleid, status_code=200
                    )
                else:
                    self.assertNotContains(
                        response, "Accès aux places réservées aux étudiants de l’" + ecoleid, status_code=200
                    )
            self.client.logout()

    @tag("ldap", "fast", "local")
    def test_LdapChangeInfo(self):
        self.assertTrue(loginUser(self.client, "testone"))
        form = {
            "first_name": "Tester",
            "last_name": "C'est la vie",
            "email": "test2@test.inpt.fr",
            "conditions_field": True,
            "update-profile": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(len(User.objects.filter(email="test2@test.inpt.fr")), 0)
        self.assertEqual(len(User.objects.filter(login_inp_net="testone")), 1)
        user = User.objects.get(login_inp_net="testone")
        self.assertEqual(user.first_name, test_user["testone"]["first_name"])
        self.assertEqual(user.last_name, test_user["testone"]["last_name"])

    @tag("ldap", "fast", "local")
    def test_LdapChangePasswd(self):
        self.assertTrue(loginUser(self.client, "testone"))
        form = {
            "old_password": "test",
            "new_password1": "passwd1",
            "new_password2": "passwd1",
            "update-password": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mail.outbox), 0)
        self.assertFalse(User.objects.get(login_inp_net="testone").has_usable_password())

    @tag("ldap", "fast", "local")
    def test_LdapAssociation(self):
        # ce test est local si il n'y a pas d'erreur
        self.assertTrue(loginUser(self.client, "testone"))
        form = {
            "login_inp_net": "testwo",
            "associate": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mail.outbox), 0)
        self.assertFalse(AssociateToken.objects.all().exists())

    @tag("ldap", "fast", "local")
    def test_LdapLogout(self):
        self.assertTrue(loginUser(self.client, "testone"))
        response = self.client.get(reverse("logout"))
        self.assertEqual(response.status_code, 302)

    @tag("exte", "fast", "local")
    def test_exteUserLogout(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        response = self.client.get(reverse("logout"))
        self.assertEqual(response.status_code, 302)

    @tag("local", "fast", "exte")
    def test_exteUserGet(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        response = self.client.get(reverse("profile"))
        self.assertContains(response, "Vous ne disposez actuellement d’aucun avantage.", status_code=200)
        self.assertTemplateUsed(response, "accounts/profile.html")

    @tag("local", "fast", "exte")
    def test_exteChangeName(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        form = {
            "first_name": "Tester",
            "last_name": "C'est la vie",
            "email": "exte@test.inpt.fr",
            "conditions_field": True,
            "update-profile": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertRedirects(response, reverse("profile"))
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(len(User.objects.filter(email="exte@test.inpt.fr")), 1)
        user = User.objects.get(email="exte@test.inpt.fr")
        self.assertEqual(user.first_name, "Tester")
        self.assertEqual(user.last_name, "C'est la vie")

    @tag("local", "fast", "exte")
    def test_exteFormChangeEmail(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        form = {
            "first_name": "test",
            "last_name": "exte",
            "email": "changemail@test.inpt.fr",
            "conditions_field": True,
            "update-profile": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertRedirects(response, reverse("profile"))
        self.assertFalse(User.objects.filter(email="changemail@test.inpt.fr").exists())
        self.assertEqual(len(User.objects.filter(email="exte@test.inpt.fr")), 1)
        user = User.objects.get(email="exte@test.inpt.fr")
        # token
        self.assertEqual(len(Token.objects.filter(usage=Token.USAGE_EMAIL_VALIDATION, user=user, used=False)), 1)
        token = Token.objects.get(usage=Token.USAGE_EMAIL_VALIDATION, user=user, used=False)
        self.assertEqual(token.payload, "changemail@test.inpt.fr")
        lientoken = reverse("validate-email", args=[token.uuid])
        # mail
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(form["email"], mail.outbox[0].to)
        self.assertIn(lientoken, mail.outbox[0].body)
        # repost to generate another token
        response = self.client.post(reverse("profile"), form)
        self.assertRedirects(response, reverse("profile"))
        self.assertFalse(User.objects.filter(email="changemail@test.inpt.fr").exists())
        self.assertEqual(len(User.objects.filter(email="exte@test.inpt.fr")), 1)
        self.assertEqual(len(Token.objects.filter(usage=Token.USAGE_EMAIL_VALIDATION, user=user, used=False)), 1)
        self.assertNotEqual(
            Token.objects.get(usage=Token.USAGE_EMAIL_VALIDATION, user=user, used=False).uuid, token.uuid
        )

    @tag("local", "fast", "exte")
    def test_exteChangePasswd(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        form = {
            "old_password": "passwd",
            "new_password1": "passwd1",
            "new_password2": "passwd1",
            "update-password": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(len(mail.outbox), 0)
        self.client.logout()
        self.assertFalse(self.client.login(email="exte@test.inpt.fr", password="passwd"))
        self.assertTrue(self.client.login(email="exte@test.inpt.fr", password="passwd1"))

    @tag("exte", "fast")
    def test_exteAssociationWithAlreadyUse(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        form = {
            "login_inp_net": "testone",
            "associate": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mail.outbox), 0)
        self.assertFalse(AssociateToken.objects.all().exists())
        self.assertFormError(
            response,
            "associateform",
            "login_inp_net",
            "Pas de compte INP-net trouvé pour ce login ou compte déjà existant sur la billetterie",
        )

    @tag("exte", "fast")
    def test_exteAssociationNotAlreadyUse(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        form = {
            "login_inp_net": "testwo",
            "associate": "",
        }
        response = self.client.post(reverse("profile"), form)
        self.assertRedirects(response, reverse("profile"))
        user = User.objects.get(email="exte@test.inpt.fr")
        self.assertEqual(
            len(
                AssociateToken.objects.filter(
                    user=user, user_ldap="testwo", validation_user=False, validation_user_ldap=False, invalide=False
                )
            ),
            1,
        )
        token = AssociateToken.objects.get(
            user=user, user_ldap="testwo", validation_user=False, validation_user_ldap=False, invalide=False
        )
        self.assertEqual(len(mail.outbox), 2)

        self.exteAssociationNotAlreadyUse_verifmailuser(
            mail.outbox[0] if user.email in mail.outbox[0].to else mail.outbox[1], token
        )
        self.exteAssociationNotAlreadyUse_verifmailldapuser(
            mail.outbox[1] if user.email in mail.outbox[0].to else mail.outbox[0], token
        )

    def exteAssociationNotAlreadyUse_verifmailuser(self, mmail, token):
        self.assertEqual(mmail.from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(token.user.email, mmail.to)
        link = reverse("associate_account") + "?part=user&token=" + token.token_user
        self.assertIn(link + "\n", mmail.body)
        self.assertIn(link + "&invalidate=True", mmail.body)

    def exteAssociationNotAlreadyUse_verifmailldapuser(self, mmail, token):
        self.assertEqual(mmail.from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(token.get_user_ldap().mail, mmail.to)
        link = reverse("associate_account") + "?part=ldap&token=" + token.token_user_ldap
        self.assertIn(link + "\n", mmail.body)
        self.assertIn(link + "&invalidate=True", mmail.body)


class changeMailTest(CustomTestCase):
    def setUp(self):
        createExteUser("exte@test.inpt.fr", "passwd", "test", "exte")

    @tag("fast", "nologin", "local")
    def test_ldapUser(self):
        createLdapUser("testone")
        token = Token(
            usage=Token.USAGE_EMAIL_VALIDATION,
            user=User.objects.get(login_inp_net="testone"),
            expire_at=timezone.now() + timedelta(hours=1),
            payload="otheremail@test.inpt.fr",
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("validate-email", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)
        self.assertEqual(User.objects.get(login_inp_net="testone").email, "testone@bde.enseeiht.fr")

    @tag("fast", "nologin", "local")
    def test_NormalUsage(self):
        token = Token(
            usage=Token.USAGE_EMAIL_VALIDATION,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() + timedelta(hours=1),
            payload="otheremail@test.inpt.fr",
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("validate-email", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)
        self.assertTrue(User.objects.filter(email="otheremail@test.inpt.fr").exists())
        self.assertFalse(User.objects.filter(email="exte@test.inpt.fr").exists())

    @tag("fast", "nologin", "local")
    def test_TokenUsed(self):
        token = Token(
            usage=Token.USAGE_EMAIL_VALIDATION,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() + timedelta(hours=1),
            payload="otheremail@test.inpt.fr",
            used=True,
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("validate-email", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)
        self.assertFalse(User.objects.filter(email="otheremail@test.inpt.fr").exists())
        self.assertTrue(User.objects.filter(email="exte@test.inpt.fr").exists())

    @tag("fast", "nologin", "local")
    def test_TokenExpired(self):
        token = Token(
            usage=Token.USAGE_EMAIL_VALIDATION,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() - timedelta(hours=1),
            payload="otheremail@test.inpt.fr",
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("validate-email", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)
        self.assertFalse(User.objects.filter(email="otheremail@test.inpt.fr").exists())
        self.assertTrue(User.objects.filter(email="exte@test.inpt.fr").exists())

    @tag("fast", "nologin", "local")
    def test_TokenEmailExisted(self):
        createExteUser("otheremail@test.inpt.fr", "passwd", "test", "exte")
        token = Token(
            usage=Token.USAGE_EMAIL_VALIDATION,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() + timedelta(hours=1),
            payload="otheremail@test.inpt.fr",
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("validate-email", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)
        self.assertEqual(len(User.objects.filter(email="otheremail@test.inpt.fr")), 1)
        self.assertTrue(User.objects.filter(email="exte@test.inpt.fr").exists())


class recoveryUserTest(CustomTestCase):
    def setUp(self):
        createLdapUser("testone")
        createExteUser("exte@test.inpt.fr", "passwd", "test", "exte")

    @tag("fast", "local", "exte")
    def test_alreadyconnected(self):
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        response = self.client.get(reverse("recovery-user"))
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)

    @tag("fast", "nologin", "local")
    def test_recoverForm(self):
        response = self.client.get(reverse("recovery-user"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "accounts/user_recovery.html")

    @tag("fast", "nologin", "local")
    def test_recoverLdapUser(self):
        form = {
            "email": "testone@bde.enseeiht.fr",
        }
        response = self.client.post(reverse("recovery-user"), form)
        self.assertRedirects(response, reverse("login"))
        self.assertEqual(len(mail.outbox), 0)

    @tag("fast", "nologin", "local")
    def test_recoverLdapUserNeverLoged(self):
        form = {
            "email": "testwo@bde.enseeiht.fr",
        }
        response = self.client.post(reverse("recovery-user"), form)
        self.assertContains(response, "Aucun compte associé à l’email spécifié n’a été trouvé.", status_code=200)
        self.assertTemplateUsed(response, "accounts/user_recovery.html")
        self.assertEqual(len(mail.outbox), 0)

    @tag("fast", "nologin", "local")
    def test_recoverNotAccountUser(self):
        form = {
            "email": "exte2@test.inpt.fr",
        }
        response = self.client.post(reverse("recovery-user"), form)
        self.assertContains(response, "Aucun compte associé à l’email spécifié n’a été trouvé.", status_code=200)
        self.assertTemplateUsed(response, "accounts/user_recovery.html")
        self.assertEqual(len(mail.outbox), 0)

    @tag("fast", "nologin", "local")
    def test_TokenUsed(self):
        token = Token(
            usage=Token.USAGE_DEFINE_PASSWORD,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() + timedelta(hours=1),
            used=True,
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("change-password", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)

    @tag("fast", "nologin", "local")
    def test_TokenExpired(self):
        token = Token(
            usage=Token.USAGE_DEFINE_PASSWORD,
            user=User.objects.get(email="exte@test.inpt.fr"),
            expire_at=timezone.now() - timedelta(hours=1),
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("change-password", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)

    @tag("fast", "nologin", "local")
    def test_TokenLdapUser(self):
        token = Token(
            usage=Token.USAGE_DEFINE_PASSWORD,
            user=User.objects.get(login_inp_net="testone"),
            expire_at=timezone.now() + timedelta(hours=1),
        )
        token.get_new_uuid()
        token.save()

        response = self.client.get(reverse("change-password", args=[token.uuid]))
        self.assertRedirects(response, reverse("profile"), fetch_redirect_response=False)
        self.assertTrue(Token.objects.get(uuid=token.uuid).used)

    @tag("fast", "exte", "local")
    def test_recoverExteUser(self):
        form = {
            "email": "exte@test.inpt.fr",
        }
        response = self.client.post(reverse("recovery-user"), form)
        self.assertRedirects(response, reverse("login"))
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(form["email"], mail.outbox[0].to)

        # token
        self.assertEqual(len(User.objects.filter(email=form["email"])), 1)
        user = User.objects.get(email=form["email"])
        self.assertEqual(len(Token.objects.filter(usage=Token.USAGE_DEFINE_PASSWORD, user=user)), 1)
        token = Token.objects.get(usage=Token.USAGE_DEFINE_PASSWORD, user=user)
        lienToken = reverse("change-password", args=[token.uuid])
        self.assertIn(lienToken, mail.outbox[0].body)

        response_2 = self.client.get(lienToken)
        self.assertEqual(response_2.status_code, 200)
        self.assertTemplateUsed(response_2, "accounts/user_password.html")

        passwd = {
            "new_password1": "test2",
            "new_password2": "test2",
        }

        # POST token
        response_3 = self.client.post(lienToken, passwd)
        self.assertRedirects(response_3, reverse("login"))

        # Vérification changement mot de passe
        self.assertTrue(self.client.login(email=form["email"], password=passwd["new_password1"]))
        self.client.logout()
        self.assertFalse(self.client.login(email=form["email"], password="passwd"))

        # Vérification des dates de suppression
        self.assertTrue(User.objects.get(email=form["email"]).mail_delete)
        self.assertGreater(User.objects.get(email=form["email"]).delete_date, timezone.now() + timedelta(days=364))


class createUserTest(CustomTestCase):
    @tag("fast", "nologin", "local")
    def test_module_accessible(self):
        response = self.client.get("/", follow=True)
        self.assertContains(response, '<a href="' + reverse("login") + '">', status_code=200)
        self.assertTemplateUsed(response, "base.html")

        response_2 = self.client.get(reverse("login"))
        self.assertEqual(response_2.status_code, 200)
        self.assertTemplateUsed(response_2, "accounts/login.html")

    @tag("fast", "local", "exte")
    def test_alreadyconnected(self):
        createExteUser("exte@test.inpt.fr", "passwd", "test", "exte")
        self.assertTrue(loginUser(self.client, "exte@test.inpt.fr"))
        response = self.client.get(reverse("create-user"))
        self.assertRedirects(response, reverse("home"), fetch_redirect_response=False)

    @tag("fast", "nologin", "local")
    def test_createAccountsAlreadyExists(self):
        createLdapUser("testone")
        form = {
            "last_name": "test",
            "first_name": "exte",
            "email": "testone@bde.enseeiht.fr",
            "conditions_field": True,
        }
        response = self.client.post(reverse("create-user"), form)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "accounts/user_creation.html")
        self.assertFormError(response, "form", "email", "Un compte existe déjà pour cette adresse email.")

    @tag("fast", "exte", "local")
    def test_newuser(self):
        # Création d'un utilisateur extérieur et connexion

        # GET subscribe
        response = self.client.get(reverse("create-user"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "accounts/user_creation.html")

        user = {
            "last_name": "test",
            "first_name": "exte",
            "email": "exte@test.inpt.fr",
            "conditions_field": True,
        }

        # POST subscribe
        response_2 = self.client.post(reverse("create-user"), user)
        self.assertRedirects(response_2, reverse("login"))

        # test envoie du mail
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertIn(user["email"], mail.outbox[0].to)

        # token
        self.assertEqual(len(User.objects.filter(email=user["email"])), 1)
        usercreate = User.objects.get(email=user["email"])
        # Vérification des dates de suppression
        self.assertFalse(usercreate.mail_delete)
        self.assertLess(usercreate.delete_date, timezone.now() + timedelta(days=8))
        self.assertGreater(usercreate.delete_date, timezone.now() + timedelta(days=6))

        self.assertEqual(len(Token.objects.filter(usage=Token.USAGE_DEFINE_PASSWORD, user=usercreate)), 1)
        token = Token.objects.get(usage=Token.USAGE_DEFINE_PASSWORD, user=usercreate)
        lienToken = reverse("validate-user", args=[token.uuid])

        self.assertIn(lienToken, mail.outbox[0].body)

        # GET token
        response_3 = self.client.get(lienToken)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response_3, "accounts/user_password.html")

        passwd = {
            "new_password1": "test",
            "new_password2": "test",
        }

        # POST token
        response_4 = self.client.post(lienToken, passwd)
        self.assertRedirects(response_4, reverse("login"))

        login = {
            "username": user["email"],
            "password": passwd["new_password1"],
        }

        # POST login
        response_5 = self.client.post(reverse("login"), login)
        self.assertRedirects(response_5, reverse("profile"))

        # Vérification changement mot de passe
        self.client.logout()
        usercreate = User.objects.get(email=user["email"])
        self.assertTrue(self.client.login(email=usercreate.email, password=passwd["new_password1"]))
        # Vérification des dates de suppression
        self.assertTrue(usercreate.mail_delete)
        self.assertGreater(usercreate.delete_date, timezone.now() + timedelta(days=364))


class deleteUserTest(CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self.userinp = User.objects.get(login_inp_net="testone")
        self.userexte = User.objects.get(email="exte@billetterie.inpt.fr")
        self.userexte.set_password("passwd")
        self.userexte.save()
        self.event = Event.objects.get(pk=1)
        self.ticket = Ticket.objects.get(pk=1)

    @tag("fast", "local")
    def test_delete_INP_user(self):
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("profile"))
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, reverse("delete_account"))
        response2 = self.client.get(reverse("delete_account"))
        self.assertRedirects(response2, reverse("profile"))

    @tag("fast", "local")
    def test_delete_exte_user(self):
        self.client.force_login(self.userexte)
        self.assertFalse(self.userexte.registrations.all().exists())
        response = self.client.get(reverse("profile"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, reverse("delete_account"))
        response2 = self.client.get(reverse("delete_account"))
        self.assertEqual(response2.status_code, 200)
        # mauvais mot de passe
        response3 = self.client.post(reverse("delete_account"), {"password": "pass"})
        self.assertEqual(response3.status_code, 200)
        self.assertTrue(User.objects.filter(pk=self.userexte.pk).exists())
        # bon mot de passe
        response4 = self.client.post(reverse("delete_account"), {"password": "passwd"})
        self.assertRedirects(response4, reverse("home"), fetch_redirect_response=False)
        self.assertFalse(User.objects.filter(pk=self.userexte.pk).exists())

    @tag("fast", "local")
    def test_delete_exte_user_with_long_past_registration(self):
        self.client.force_login(self.userexte)
        self.assertFalse(self.userexte.registrations.all().exists())
        reg = Registration(ticket=self.ticket, owner=self.userexte, paid=True)
        reg.save()
        self.event.start_date = timezone.now() - timedelta(days=60)
        self.event.end_date = timezone.now() - timedelta(days=59)
        self.event.save()
        response = self.client.get(reverse("profile"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, reverse("delete_account"))
        response2 = self.client.get(reverse("delete_account"))
        self.assertEqual(response2.status_code, 200)
        response3 = self.client.post(reverse("delete_account"), {"password": "passwd"})
        self.assertRedirects(response3, reverse("home"), fetch_redirect_response=False)
        self.assertFalse(User.objects.filter(pk=self.userexte.pk).exists())
        regs = Registration.objects.filter(pk=reg.pk)
        self.assertEqual(len(regs), 1)
        self.assertEqual(regs.first().owner, User.get_anonyme_user())

    @tag("fast", "local")
    def test_delete_exte_user_with_short_past_registration(self):
        self.client.force_login(self.userexte)
        self.assertFalse(self.userexte.registrations.all().exists())
        reg = Registration(ticket=self.ticket, owner=self.userexte, paid=True)
        reg.save()
        self.event.start_date = timezone.now() - timedelta(days=11)
        self.event.end_date = timezone.now() - timedelta(days=10)
        self.event.save()
        response = self.client.get(reverse("profile"))
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, reverse("delete_account"))
        response2 = self.client.get(reverse("delete_account"))
        self.assertRedirects(response2, reverse("profile"))

    @tag("fast", "local")
    def test_delete_exte_user_with_future_registration(self):
        self.client.force_login(self.userexte)
        self.assertFalse(self.userexte.registrations.all().exists())
        reg = Registration(ticket=self.ticket, owner=self.userexte, paid=True)
        reg.save()
        self.event.start_date = timezone.now() + timedelta(days=13)
        self.event.end_date = timezone.now() + timedelta(days=15)
        self.event.save()
        response = self.client.get(reverse("profile"))
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, reverse("delete_account"))
        response2 = self.client.get(reverse("delete_account"))
        self.assertRedirects(response2, reverse("profile"))


class ExportDataTest(CustomTestCase):

    fixtures = ["test_guichet_users", "test_guichet_views"]

    def setUp(self):
        self._file_todelete = []
        self.userinp = User.objects.get(login_inp_net="testone")
        self.userexte = User.objects.get(email="exte@billetterie.inpt.fr")
        self.userexte.set_password("passwd")
        self.userexte.save()

    def tearDown(self):
        for i in self._file_todelete:
            os.remove(i)
        path_export = os.path.join(settings.MEDIA_ROOT, "protected", "export_archive")
        if os.path.isdir(path_export):
            for i in os.listdir(path_export):
                os.remove(os.path.join(path_export, i))

    def hardsetup(self):
        self.userinp.charte_version = 0
        self.userinp.charte_date = timezone.now()
        self.userinp.is_superuser = True
        self.userinp.save()
        event = Event.objects.get(pk=1)
        GroupManager(
            event=event,
            access=Manager.ENTRY_MANAGER,
            group=Group.objects.get(pk=1),
            bureau_only=True,
        ).save()
        GroupManager(
            event=event,
            access=Manager.FULL_VIEW,
            group=Group.objects.get(pk=2),
            bureau_only=False,
        ).save()
        ticket = Ticket.objects.get(pk=1)
        opt = Option(
            ticket=ticket,
            name="Test",
            price=10,
            group_name="Group",
            group=1,
        )
        opt.save()
        info1 = Info_Sup_Int(
            ticket=ticket,
            type_info=Info_Sup.INFO_INT,
            name="Info int",
        )
        info1.save()
        info2 = Info_Sup_File(
            ticket=ticket,
            type_info=Info_Sup.INFO_FILE,
            name="Info file",
            file_name="file_export",
        )
        info2.save()
        reg = Registration(
            ticket=ticket,
            owner=self.userinp,
            paid=True,
            payment_method=Registration.PAYMENT_PAYPAL,
            payment_date=timezone.now(),
            opposed=True,
            used=True,
            used_date=timezone.now(),
        )
        reg.save()
        reg.options.add(opt)
        reg.save()
        info1_sup = info1.received.get(registration=reg).get_sup_type()
        info1_sup.state = info1_sup.STATE_ACCEPTED
        info1_sup.value = 10
        info1_sup.save()
        info2_sup = info2.received.get(registration=reg).get_sup_type()
        info2_sup.state = info2_sup.STATE_ACCEPTED
        path = os.path.join(settings.BASE_DIR, "guichet", "tests", "ressources", "ticket_template.png")
        with open(path, "rb") as f:
            info2_sup.value = UploadedFile(f, "template_test.png")
            info2_sup.save()
            self._file_todelete.append(os.path.join(settings.MEDIA_ROOT, info2_sup.value.path))

        PayPalIPN(
            query="querypaypal",
            flag=False,
            payment_status="Completed",
            payment_date=timezone.now(),
            mc_currency="EUR",
            mc_gross=500,
            payer_email="admin@billetterie.inpt.fr",
            invoice=settings.PAYPAL_INVOICE_PREFIX + str(reg.pk),
        ).save()

    @tag("fast", "local")
    def test_exportExteBadPasswd(self):
        self.client.force_login(self.userexte)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "badpassword"})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateUsed(response2, "accounts/access_data_form.html")

    @tag("fast", "local")
    def test_exportExte(self):
        self.client.force_login(self.userexte)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "passwd", "access": ""})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateUsed(response2, "accounts/access_data.html")

    @tag("low", "ldap")
    def test_exportInpBadPasswd(self):
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "badpassword"})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateUsed(response2, "accounts/access_data_form.html")

    @tag("low", "ldap")
    def test_exportInp(self):
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "test", "access": ""})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateUsed(response2, "accounts/access_data.html")

    @tag("low", "local")
    def test_exportInpAll(self):
        self.hardsetup()
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "test", "access": ""})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateUsed(response2, "accounts/access_data.html")

    @tag("low", "local")
    def test_exportInpAllZip(self):
        self.hardsetup()
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "test", "zip": ""})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateNotUsed(response2, "accounts/access_data.html")
        self.assertTemplateNotUsed(response2, "accounts/access_data_form.html")

    @tag("low", "local")
    def test_exportInpAllTar(self):
        self.hardsetup()
        self.client.force_login(self.userinp)
        response = self.client.get(reverse("access_user_data"))
        self.assertEqual(response.status_code, 200)

        response2 = self.client.post(reverse("access_user_data"), data={"password": "test", "tar": ""})
        self.assertEqual(response2.status_code, 200)
        self.assertTemplateNotUsed(response2, "accounts/access_data.html")
        self.assertTemplateNotUsed(response2, "accounts/access_data_form.html")
